
from django.db.models import Q
from django.urls import resolve, Resolver404

from seo.models import PageMeta


def _get_url_full_name(request):

    name = ''

    try:
        url = resolve(request.path_info)
    except Resolver404:
        return ''

    if url.url_name is None:
        return ''

    if url.namespaces:
        name = ':'.join(url.namespaces) + ':'

    name += url.url_name

    return name


def _get_current_url(request):

    path = request.path

    if path.startswith('/{}/'.format(request.LANGUAGE_CODE)):
        return path[3:]

    return path


def get_page_meta(request, context):

    try:
        page_meta = PageMeta.objects.get(
            Q(url=_get_current_url(request)) |
            Q(url=_get_url_full_name(request))
        )

        page_meta.compile(context)

        return page_meta

    except PageMeta.DoesNotExist:
        return None




from django.template import Library

from ..lib import get_page_meta


register = Library()


@register.simple_tag(takes_context=True, name='get_page_meta')
def get_page_meta_tag(context):
    return get_page_meta(context.request, context)




import re

from django.db import models
from django.utils.translation import ugettext_lazy as _


PLACEHOLDER_REGEX = re.compile(r'\{\{\s\s*(\S*)\s*\s\}\}')
PLACEHOLDER_TEMPLATE = '{{ {} }}'


class PageMeta(models.Model):

    META_ROBOTS_CHOICES = (
        ('index, follow', _('Index, Follow')),
        ('noindex, nofollow', _('No Index, No Follow')),
        ('index, nofollow', _('Index, No Follow')),
        ('noindex, follow', _('No Index, Follow')),
    )

    url = models.CharField(
        _('URL'), max_length=255, blank=False, unique=True, db_index=True)

    title = models.CharField(_('Title'), max_length=68, blank=False)

    keywords = models.CharField(_('Keywords'), max_length=100, blank=True)

    description = models.CharField(
        _('Description'), max_length=155, blank=True)

    breadcrumb = models.CharField(_('Breadcrumb'), max_length=100, blank=True)

    header = models.CharField(_('Header'), max_length=100, blank=True)

    robots = models.CharField(
        _('Robots'), max_length=30, blank=True, default='index, follow',
        choices=META_ROBOTS_CHOICES)

    compile_fields = [
        'title', 'keywords', 'description', 'breadcrumb', 'header']

    @staticmethod
    def _get_context_value(context, path):

        value = None

        for key in path.split('.'):
            if value is None:
                value = context.get(key)
            else:
                if isinstance(value, dict):
                    value = value.get(key)
                else:
                    value = getattr(value, key)

        if callable(value):
            value = value()

        return value

    def _render_tag(self, text, context):

        for path in re.findall(PLACEHOLDER_REGEX, text):

            try:
                value = self._get_context_value(context, path)
            except Exception as e:
                value = ''

            try:
                text = text.replace(PLACEHOLDER_TEMPLATE.format(path), value)
            except TypeError:
                pass

        return text

    def compile(self, context):
        for field in self.compile_fields:
            value = self._render_tag(getattr(self, field), context)
            setattr(self, 'printable_' + field, value)

    def __str__(self):
        return '{} - {}'.format(self.url, self.title)

    class Meta:
        verbose_name = _('Page meta')
        verbose_name_plural = _('Pages meta')




from modeltranslation.translator import register, TranslationOptions

from seo.models import PageMeta


@register(PageMeta)
class PageMetaTranslationOptions(TranslationOptions):

    fields = ('title', 'keywords', 'description', 'breadcrumb', 'header', )



# Generated by Django 2.0.6 on 2018-06-20 07:14

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PageMeta',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(db_index=True, max_length=255, unique=True, verbose_name='URL')),
                ('title', models.CharField(max_length=68, verbose_name='Title')),
                ('keywords', models.CharField(blank=True, max_length=100, verbose_name='Keywords')),
                ('description', models.CharField(blank=True, max_length=155, verbose_name='Description')),
                ('breadcrumb', models.CharField(blank=True, max_length=100, verbose_name='Breadcrumb')),
                ('header', models.CharField(blank=True, max_length=100, verbose_name='Header')),
                ('robots', models.CharField(blank=True, choices=[('index, follow', 'Index, Follow'), ('noindex, nofollow', 'No Index, No Follow'), ('index, nofollow', 'Index, No Follow'), ('noindex, follow', 'No Index, Follow')], default='index, follow', max_length=30, verbose_name='Robots')),
            ],
            options={
                'verbose_name': 'Page meta',
                'verbose_name_plural': 'Pages meta',
            },
        ),
    ]




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class SeoConfig(AppConfig):
    name = 'seo'
    verbose_name = _("SEO")


default_app_config = 'seo.SeoConfig'

__version__ = '3.3'




from importlib import import_module

from django.apps import apps
from django.contrib import admin

from seo.models import PageMeta


def _get_page_meta_admin_base_class():

    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TranslationAdmin

    return admin.ModelAdmin


class PageMetaAdmin(_get_page_meta_admin_base_class()):

    list_display = ['url', 'title', 'robots']
    list_editable = ['robots']
    list_filter = ['robots']
    search_fields = ['url', 'title']


admin.site.register(PageMeta, PageMetaAdmin)
